# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._stripe_service import StripeService
from stripe.v2.tax._automatic_rule_service import AutomaticRuleService


class TaxService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.automatic_rules = AutomaticRuleService(self._requestor)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.params.v2.billing.collection_settings._version_list_params import (
    VersionListParams as VersionListParams,
)
from stripe.params.v2.billing.collection_settings._version_retrieve_params import (
    VersionRetrieveParams as VersionRetrieveParams,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from typing import Dict, List, Union
from typing_extensions import Literal, NotRequired, TypedDict


class PaymentIntentIncrementAuthorizationParams(RequestOptions):
    amount: int
    """
    The updated total amount that you intend to collect from the cardholder. This amount must be greater than the currently authorized amount.
    """
    amount_details: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetails"
    ]
    """
    Provides industry-specific information about the amount.
    """
    application_fee_amount: NotRequired[int]
    """
    The amount of the application fee (if any) that will be requested to be applied to the payment and transferred to the application owner's Stripe account. The amount of the application fee collected will be capped at the total amount captured. For more information, see the PaymentIntents [use case for connected accounts](https://stripe.com/docs/payments/connected-accounts).
    """
    description: NotRequired[str]
    """
    An arbitrary string attached to the object. Often useful for displaying to users.
    """
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """
    hooks: NotRequired["PaymentIntentIncrementAuthorizationParamsHooks"]
    """
    Automations to be run during the PaymentIntent lifecycle
    """
    metadata: NotRequired[Dict[str, str]]
    """
    Set of [key-value pairs](https://stripe.com/docs/api/metadata) that you can attach to an object. This can be useful for storing additional information about the object in a structured format. Individual keys can be unset by posting an empty value to them. All keys can be unset by posting an empty value to `metadata`.
    """
    payment_details: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsPaymentDetails"
    ]
    """
    Provides industry-specific information about the charge.
    """
    payment_method_options: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsPaymentMethodOptions"
    ]
    """
    Payment method-specific configuration for this PaymentIntent.
    """
    statement_descriptor: NotRequired[str]
    """
    Text that appears on the customer's statement as the statement descriptor for a non-card or card charge. This value overrides the account's default statement descriptor. For information about requirements, including the 22-character limit, see [the Statement Descriptor docs](https://docs.stripe.com/get-started/account/statement-descriptors).
    """
    transfer_data: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsTransferData"
    ]
    """
    The parameters used to automatically create a transfer after the payment is captured.
    Learn more about the [use case for connected accounts](https://stripe.com/docs/payments/connected-accounts).
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetails(TypedDict):
    discount_amount: NotRequired["Literal['']|int"]
    """
    The total discount applied on the transaction.
    """
    line_items: NotRequired[
        "Literal['']|List[PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItem]"
    ]
    """
    A list of line items, each containing information about a product in the PaymentIntent. There is a maximum of 100 line items.
    """
    shipping: NotRequired[
        "Literal['']|PaymentIntentIncrementAuthorizationParamsAmountDetailsShipping"
    ]
    """
    Contains information about the shipping portion of the amount.
    """
    tax: NotRequired[
        "Literal['']|PaymentIntentIncrementAuthorizationParamsAmountDetailsTax"
    ]
    """
    Contains information about the tax portion of the amount.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItem(
    TypedDict
):
    discount_amount: NotRequired[int]
    """
    The amount an item was discounted for. Positive integer.
    """
    payment_method_options: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptions"
    ]
    """
    Payment method-specific information for line items.
    """
    product_code: NotRequired[str]
    """
    Unique identifier of the product. At most 12 characters long.
    """
    product_name: str
    """
    Name of the product. At most 100 characters long.
    """
    quantity: int
    """
    Number of items of the product. Positive integer.
    """
    tax: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemTax"
    ]
    """
    Contains information about the tax on the item.
    """
    unit_cost: int
    """
    Cost of the product. Non-negative integer.
    """
    unit_of_measure: NotRequired[str]
    """
    A unit of measure for the line item, such as gallons, feet, meters, etc.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptions(
    TypedDict,
):
    card: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsCard"
    ]
    """
    This sub-hash contains line item details that are specific to `card` payment method."
    """
    card_present: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsCardPresent"
    ]
    """
    This sub-hash contains line item details that are specific to `card_present` payment method."
    """
    klarna: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsKlarna"
    ]
    """
    This sub-hash contains line item details that are specific to `klarna` payment method."
    """
    paypal: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsPaypal"
    ]
    """
    This sub-hash contains line item details that are specific to `paypal` payment method."
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsCard(
    TypedDict,
):
    commodity_code: NotRequired[str]
    """
    Identifier that categorizes the items being purchased using a standardized commodity scheme such as (but not limited to) UNSPSC, NAICS, NAPCS, etc.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsCardPresent(
    TypedDict,
):
    commodity_code: NotRequired[str]
    """
    Identifier that categorizes the items being purchased using a standardized commodity scheme such as (but not limited to) UNSPSC, NAICS, NAPCS, etc.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsKlarna(
    TypedDict,
):
    image_url: NotRequired[str]
    """
    URL to an image for the product. Max length, 4096 characters.
    """
    product_url: NotRequired[str]
    """
    URL to the product page. Max length, 4096 characters.
    """
    subscription_reference: NotRequired[str]
    """
    Reference for the subscription this line item is for.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemPaymentMethodOptionsPaypal(
    TypedDict,
):
    category: NotRequired[
        Literal["digital_goods", "donation", "physical_goods"]
    ]
    """
    Type of the line item.
    """
    description: NotRequired[str]
    """
    Description of the line item.
    """
    sold_by: NotRequired[str]
    """
    The Stripe account ID of the connected account that sells the item.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsLineItemTax(
    TypedDict,
):
    total_tax_amount: int
    """
    The total tax on an item. Non-negative integer.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsShipping(
    TypedDict
):
    amount: NotRequired["Literal['']|int"]
    """
    Portion of the amount that is for shipping.
    """
    from_postal_code: NotRequired["Literal['']|str"]
    """
    The postal code that represents the shipping source.
    """
    to_postal_code: NotRequired["Literal['']|str"]
    """
    The postal code that represents the shipping destination.
    """


class PaymentIntentIncrementAuthorizationParamsAmountDetailsTax(TypedDict):
    total_tax_amount: int
    """
    Total portion of the amount that is for tax.
    """


class PaymentIntentIncrementAuthorizationParamsHooks(TypedDict):
    inputs: NotRequired["PaymentIntentIncrementAuthorizationParamsHooksInputs"]
    """
    Arguments passed in automations
    """


class PaymentIntentIncrementAuthorizationParamsHooksInputs(TypedDict):
    tax: NotRequired["PaymentIntentIncrementAuthorizationParamsHooksInputsTax"]
    """
    Tax arguments for automations
    """


class PaymentIntentIncrementAuthorizationParamsHooksInputsTax(TypedDict):
    calculation: Union[Literal[""], str]
    """
    The [TaxCalculation](https://stripe.com/docs/api/tax/calculations) id
    """


class PaymentIntentIncrementAuthorizationParamsPaymentDetails(TypedDict):
    customer_reference: NotRequired["Literal['']|str"]
    """
    Some customers might be required by their company or organization to provide this information. If so, provide this value. Otherwise you can ignore this field.
    """
    order_reference: NotRequired["Literal['']|str"]
    """
    A unique value assigned by the business to identify the transaction.
    """


class PaymentIntentIncrementAuthorizationParamsPaymentMethodOptions(TypedDict):
    card: NotRequired[
        "PaymentIntentIncrementAuthorizationParamsPaymentMethodOptionsCard"
    ]
    """
    Configuration for any card payments attempted on this PaymentIntent.
    """


class PaymentIntentIncrementAuthorizationParamsPaymentMethodOptionsCard(
    TypedDict,
):
    request_partial_authorization: NotRequired[
        Literal["if_available", "never"]
    ]
    """
    Request partial authorization on this PaymentIntent.
    """


class PaymentIntentIncrementAuthorizationParamsTransferData(TypedDict):
    amount: NotRequired[int]
    """
    The amount that will be transferred automatically when a charge succeeds.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing import List
from typing_extensions import NotRequired, TypedDict


class CustomerPaymentSourceRetrieveParams(TypedDict):
    expand: NotRequired[List[str]]
    """
    Specifies which fields in the response should be expanded.
    """



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._list_object import ListObject
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.billing._meter import Meter
from stripe.billing._meter_event_summary_service import (
    MeterEventSummaryService,
)
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.billing._meter_create_params import MeterCreateParams
    from stripe.params.billing._meter_deactivate_params import (
        MeterDeactivateParams,
    )
    from stripe.params.billing._meter_list_params import MeterListParams
    from stripe.params.billing._meter_reactivate_params import (
        MeterReactivateParams,
    )
    from stripe.params.billing._meter_retrieve_params import (
        MeterRetrieveParams,
    )
    from stripe.params.billing._meter_update_params import MeterUpdateParams


class MeterService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.event_summaries = MeterEventSummaryService(self._requestor)

    def list(
        self,
        params: Optional["MeterListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[Meter]:
        """
        Retrieve a list of billing meters.
        """
        return cast(
            ListObject[Meter],
            self._request(
                "get",
                "/v1/billing/meters",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["MeterListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[Meter]:
        """
        Retrieve a list of billing meters.
        """
        return cast(
            ListObject[Meter],
            await self._request_async(
                "get",
                "/v1/billing/meters",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "MeterCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        Creates a billing meter.
        """
        return cast(
            Meter,
            self._request(
                "post",
                "/v1/billing/meters",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "MeterCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        Creates a billing meter.
        """
        return cast(
            Meter,
            await self._request_async(
                "post",
                "/v1/billing/meters",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["MeterRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        Retrieves a billing meter given an ID.
        """
        return cast(
            Meter,
            self._request(
                "get",
                "/v1/billing/meters/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["MeterRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        Retrieves a billing meter given an ID.
        """
        return cast(
            Meter,
            await self._request_async(
                "get",
                "/v1/billing/meters/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["MeterUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        Updates a billing meter.
        """
        return cast(
            Meter,
            self._request(
                "post",
                "/v1/billing/meters/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["MeterUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        Updates a billing meter.
        """
        return cast(
            Meter,
            await self._request_async(
                "post",
                "/v1/billing/meters/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def deactivate(
        self,
        id: str,
        params: Optional["MeterDeactivateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        When a meter is deactivated, no more meter events will be accepted for this meter. You can't attach a deactivated meter to a price.
        """
        return cast(
            Meter,
            self._request(
                "post",
                "/v1/billing/meters/{id}/deactivate".format(
                    id=sanitize_id(id)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def deactivate_async(
        self,
        id: str,
        params: Optional["MeterDeactivateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        When a meter is deactivated, no more meter events will be accepted for this meter. You can't attach a deactivated meter to a price.
        """
        return cast(
            Meter,
            await self._request_async(
                "post",
                "/v1/billing/meters/{id}/deactivate".format(
                    id=sanitize_id(id)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def reactivate(
        self,
        id: str,
        params: Optional["MeterReactivateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        When a meter is reactivated, events for this meter can be accepted and you can attach the meter to a price.
        """
        return cast(
            Meter,
            self._request(
                "post",
                "/v1/billing/meters/{id}/reactivate".format(
                    id=sanitize_id(id)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def reactivate_async(
        self,
        id: str,
        params: Optional["MeterReactivateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Meter:
        """
        When a meter is reactivated, events for this meter can be accepted and you can attach the meter to a price.
        """
        return cast(
            Meter,
            await self._request_async(
                "post",
                "/v1/billing/meters/{id}/reactivate".format(
                    id=sanitize_id(id)
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.params.treasury._credit_reversal_create_params import (
    CreditReversalCreateParams as CreditReversalCreateParams,
)
from stripe.params.treasury._credit_reversal_list_params import (
    CreditReversalListParams as CreditReversalListParams,
)
from stripe.params.treasury._credit_reversal_retrieve_params import (
    CreditReversalRetrieveParams as CreditReversalRetrieveParams,
)
from stripe.params.treasury._debit_reversal_create_params import (
    DebitReversalCreateParams as DebitReversalCreateParams,
)
from stripe.params.treasury._debit_reversal_list_params import (
    DebitReversalListParams as DebitReversalListParams,
)
from stripe.params.treasury._debit_reversal_retrieve_params import (
    DebitReversalRetrieveParams as DebitReversalRetrieveParams,
)
from stripe.params.treasury._financial_account_close_params import (
    FinancialAccountCloseParams as FinancialAccountCloseParams,
    FinancialAccountCloseParamsForwardingSettings as FinancialAccountCloseParamsForwardingSettings,
)
from stripe.params.treasury._financial_account_create_params import (
    FinancialAccountCreateParams as FinancialAccountCreateParams,
    FinancialAccountCreateParamsFeatures as FinancialAccountCreateParamsFeatures,
    FinancialAccountCreateParamsFeaturesCardIssuing as FinancialAccountCreateParamsFeaturesCardIssuing,
    FinancialAccountCreateParamsFeaturesDepositInsurance as FinancialAccountCreateParamsFeaturesDepositInsurance,
    FinancialAccountCreateParamsFeaturesFinancialAddresses as FinancialAccountCreateParamsFeaturesFinancialAddresses,
    FinancialAccountCreateParamsFeaturesFinancialAddressesAba as FinancialAccountCreateParamsFeaturesFinancialAddressesAba,
    FinancialAccountCreateParamsFeaturesInboundTransfers as FinancialAccountCreateParamsFeaturesInboundTransfers,
    FinancialAccountCreateParamsFeaturesInboundTransfersAch as FinancialAccountCreateParamsFeaturesInboundTransfersAch,
    FinancialAccountCreateParamsFeaturesIntraStripeFlows as FinancialAccountCreateParamsFeaturesIntraStripeFlows,
    FinancialAccountCreateParamsFeaturesOutboundPayments as FinancialAccountCreateParamsFeaturesOutboundPayments,
    FinancialAccountCreateParamsFeaturesOutboundPaymentsAch as FinancialAccountCreateParamsFeaturesOutboundPaymentsAch,
    FinancialAccountCreateParamsFeaturesOutboundPaymentsUsDomesticWire as FinancialAccountCreateParamsFeaturesOutboundPaymentsUsDomesticWire,
    FinancialAccountCreateParamsFeaturesOutboundTransfers as FinancialAccountCreateParamsFeaturesOutboundTransfers,
    FinancialAccountCreateParamsFeaturesOutboundTransfersAch as FinancialAccountCreateParamsFeaturesOutboundTransfersAch,
    FinancialAccountCreateParamsFeaturesOutboundTransfersUsDomesticWire as FinancialAccountCreateParamsFeaturesOutboundTransfersUsDomesticWire,
    FinancialAccountCreateParamsPlatformRestrictions as FinancialAccountCreateParamsPlatformRestrictions,
)
from stripe.params.treasury._financial_account_features_retrieve_params import (
    FinancialAccountFeaturesRetrieveParams as FinancialAccountFeaturesRetrieveParams,
)
from stripe.params.treasury._financial_account_features_update_params import (
    FinancialAccountFeaturesUpdateParams as FinancialAccountFeaturesUpdateParams,
    FinancialAccountFeaturesUpdateParamsCardIssuing as FinancialAccountFeaturesUpdateParamsCardIssuing,
    FinancialAccountFeaturesUpdateParamsDepositInsurance as FinancialAccountFeaturesUpdateParamsDepositInsurance,
    FinancialAccountFeaturesUpdateParamsFinancialAddresses as FinancialAccountFeaturesUpdateParamsFinancialAddresses,
    FinancialAccountFeaturesUpdateParamsFinancialAddressesAba as FinancialAccountFeaturesUpdateParamsFinancialAddressesAba,
    FinancialAccountFeaturesUpdateParamsInboundTransfers as FinancialAccountFeaturesUpdateParamsInboundTransfers,
    FinancialAccountFeaturesUpdateParamsInboundTransfersAch as FinancialAccountFeaturesUpdateParamsInboundTransfersAch,
    FinancialAccountFeaturesUpdateParamsIntraStripeFlows as FinancialAccountFeaturesUpdateParamsIntraStripeFlows,
    FinancialAccountFeaturesUpdateParamsOutboundPayments as FinancialAccountFeaturesUpdateParamsOutboundPayments,
    FinancialAccountFeaturesUpdateParamsOutboundPaymentsAch as FinancialAccountFeaturesUpdateParamsOutboundPaymentsAch,
    FinancialAccountFeaturesUpdateParamsOutboundPaymentsUsDomesticWire as FinancialAccountFeaturesUpdateParamsOutboundPaymentsUsDomesticWire,
    FinancialAccountFeaturesUpdateParamsOutboundTransfers as FinancialAccountFeaturesUpdateParamsOutboundTransfers,
    FinancialAccountFeaturesUpdateParamsOutboundTransfersAch as FinancialAccountFeaturesUpdateParamsOutboundTransfersAch,
    FinancialAccountFeaturesUpdateParamsOutboundTransfersUsDomesticWire as FinancialAccountFeaturesUpdateParamsOutboundTransfersUsDomesticWire,
)
from stripe.params.treasury._financial_account_list_params import (
    FinancialAccountListParams as FinancialAccountListParams,
    FinancialAccountListParamsCreated as FinancialAccountListParamsCreated,
)
from stripe.params.treasury._financial_account_modify_params import (
    FinancialAccountModifyParams as FinancialAccountModifyParams,
    FinancialAccountModifyParamsFeatures as FinancialAccountModifyParamsFeatures,
    FinancialAccountModifyParamsFeaturesCardIssuing as FinancialAccountModifyParamsFeaturesCardIssuing,
    FinancialAccountModifyParamsFeaturesDepositInsurance as FinancialAccountModifyParamsFeaturesDepositInsurance,
    FinancialAccountModifyParamsFeaturesFinancialAddresses as FinancialAccountModifyParamsFeaturesFinancialAddresses,
    FinancialAccountModifyParamsFeaturesFinancialAddressesAba as FinancialAccountModifyParamsFeaturesFinancialAddressesAba,
    FinancialAccountModifyParamsFeaturesInboundTransfers as FinancialAccountModifyParamsFeaturesInboundTransfers,
    FinancialAccountModifyParamsFeaturesInboundTransfersAch as FinancialAccountModifyParamsFeaturesInboundTransfersAch,
    FinancialAccountModifyParamsFeaturesIntraStripeFlows as FinancialAccountModifyParamsFeaturesIntraStripeFlows,
    FinancialAccountModifyParamsFeaturesOutboundPayments as FinancialAccountModifyParamsFeaturesOutboundPayments,
    FinancialAccountModifyParamsFeaturesOutboundPaymentsAch as FinancialAccountModifyParamsFeaturesOutboundPaymentsAch,
    FinancialAccountModifyParamsFeaturesOutboundPaymentsUsDomesticWire as FinancialAccountModifyParamsFeaturesOutboundPaymentsUsDomesticWire,
    FinancialAccountModifyParamsFeaturesOutboundTransfers as FinancialAccountModifyParamsFeaturesOutboundTransfers,
    FinancialAccountModifyParamsFeaturesOutboundTransfersAch as FinancialAccountModifyParamsFeaturesOutboundTransfersAch,
    FinancialAccountModifyParamsFeaturesOutboundTransfersUsDomesticWire as FinancialAccountModifyParamsFeaturesOutboundTransfersUsDomesticWire,
    FinancialAccountModifyParamsForwardingSettings as FinancialAccountModifyParamsForwardingSettings,
    FinancialAccountModifyParamsPlatformRestrictions as FinancialAccountModifyParamsPlatformRestrictions,
)
from stripe.params.treasury._financial_account_retrieve_features_params import (
    FinancialAccountRetrieveFeaturesParams as FinancialAccountRetrieveFeaturesParams,
)
from stripe.params.treasury._financial_account_retrieve_params import (
    FinancialAccountRetrieveParams as FinancialAccountRetrieveParams,
)
from stripe.params.treasury._financial_account_update_features_params import (
    FinancialAccountUpdateFeaturesParams as FinancialAccountUpdateFeaturesParams,
    FinancialAccountUpdateFeaturesParamsCardIssuing as FinancialAccountUpdateFeaturesParamsCardIssuing,
    FinancialAccountUpdateFeaturesParamsDepositInsurance as FinancialAccountUpdateFeaturesParamsDepositInsurance,
    FinancialAccountUpdateFeaturesParamsFinancialAddresses as FinancialAccountUpdateFeaturesParamsFinancialAddresses,
    FinancialAccountUpdateFeaturesParamsFinancialAddressesAba as FinancialAccountUpdateFeaturesParamsFinancialAddressesAba,
    FinancialAccountUpdateFeaturesParamsInboundTransfers as FinancialAccountUpdateFeaturesParamsInboundTransfers,
    FinancialAccountUpdateFeaturesParamsInboundTransfersAch as FinancialAccountUpdateFeaturesParamsInboundTransfersAch,
    FinancialAccountUpdateFeaturesParamsIntraStripeFlows as FinancialAccountUpdateFeaturesParamsIntraStripeFlows,
    FinancialAccountUpdateFeaturesParamsOutboundPayments as FinancialAccountUpdateFeaturesParamsOutboundPayments,
    FinancialAccountUpdateFeaturesParamsOutboundPaymentsAch as FinancialAccountUpdateFeaturesParamsOutboundPaymentsAch,
    FinancialAccountUpdateFeaturesParamsOutboundPaymentsUsDomesticWire as FinancialAccountUpdateFeaturesParamsOutboundPaymentsUsDomesticWire,
    FinancialAccountUpdateFeaturesParamsOutboundTransfers as FinancialAccountUpdateFeaturesParamsOutboundTransfers,
    FinancialAccountUpdateFeaturesParamsOutboundTransfersAch as FinancialAccountUpdateFeaturesParamsOutboundTransfersAch,
    FinancialAccountUpdateFeaturesParamsOutboundTransfersUsDomesticWire as FinancialAccountUpdateFeaturesParamsOutboundTransfersUsDomesticWire,
)
from stripe.params.treasury._financial_account_update_params import (
    FinancialAccountUpdateParams as FinancialAccountUpdateParams,
    FinancialAccountUpdateParamsFeatures as FinancialAccountUpdateParamsFeatures,
    FinancialAccountUpdateParamsFeaturesCardIssuing as FinancialAccountUpdateParamsFeaturesCardIssuing,
    FinancialAccountUpdateParamsFeaturesDepositInsurance as FinancialAccountUpdateParamsFeaturesDepositInsurance,
    FinancialAccountUpdateParamsFeaturesFinancialAddresses as FinancialAccountUpdateParamsFeaturesFinancialAddresses,
    FinancialAccountUpdateParamsFeaturesFinancialAddressesAba as FinancialAccountUpdateParamsFeaturesFinancialAddressesAba,
    FinancialAccountUpdateParamsFeaturesInboundTransfers as FinancialAccountUpdateParamsFeaturesInboundTransfers,
    FinancialAccountUpdateParamsFeaturesInboundTransfersAch as FinancialAccountUpdateParamsFeaturesInboundTransfersAch,
    FinancialAccountUpdateParamsFeaturesIntraStripeFlows as FinancialAccountUpdateParamsFeaturesIntraStripeFlows,
    FinancialAccountUpdateParamsFeaturesOutboundPayments as FinancialAccountUpdateParamsFeaturesOutboundPayments,
    FinancialAccountUpdateParamsFeaturesOutboundPaymentsAch as FinancialAccountUpdateParamsFeaturesOutboundPaymentsAch,
    FinancialAccountUpdateParamsFeaturesOutboundPaymentsUsDomesticWire as FinancialAccountUpdateParamsFeaturesOutboundPaymentsUsDomesticWire,
    FinancialAccountUpdateParamsFeaturesOutboundTransfers as FinancialAccountUpdateParamsFeaturesOutboundTransfers,
    FinancialAccountUpdateParamsFeaturesOutboundTransfersAch as FinancialAccountUpdateParamsFeaturesOutboundTransfersAch,
    FinancialAccountUpdateParamsFeaturesOutboundTransfersUsDomesticWire as FinancialAccountUpdateParamsFeaturesOutboundTransfersUsDomesticWire,
    FinancialAccountUpdateParamsForwardingSettings as FinancialAccountUpdateParamsForwardingSettings,
    FinancialAccountUpdateParamsPlatformRestrictions as FinancialAccountUpdateParamsPlatformRestrictions,
)
from stripe.params.treasury._inbound_transfer_cancel_params import (
    InboundTransferCancelParams as InboundTransferCancelParams,
)
from stripe.params.treasury._inbound_transfer_create_params import (
    InboundTransferCreateParams as InboundTransferCreateParams,
)
from stripe.params.treasury._inbound_transfer_fail_params import (
    InboundTransferFailParams as InboundTransferFailParams,
    InboundTransferFailParamsFailureDetails as InboundTransferFailParamsFailureDetails,
)
from stripe.params.treasury._inbound_transfer_list_params import (
    InboundTransferListParams as InboundTransferListParams,
)
from stripe.params.treasury._inbound_transfer_retrieve_params import (
    InboundTransferRetrieveParams as InboundTransferRetrieveParams,
)
from stripe.params.treasury._inbound_transfer_return_inbound_transfer_params import (
    InboundTransferReturnInboundTransferParams as InboundTransferReturnInboundTransferParams,
)
from stripe.params.treasury._inbound_transfer_succeed_params import (
    InboundTransferSucceedParams as InboundTransferSucceedParams,
)
from stripe.params.treasury._outbound_payment_cancel_params import (
    OutboundPaymentCancelParams as OutboundPaymentCancelParams,
)
from stripe.params.treasury._outbound_payment_create_params import (
    OutboundPaymentCreateParams as OutboundPaymentCreateParams,
    OutboundPaymentCreateParamsDestinationPaymentMethodData as OutboundPaymentCreateParamsDestinationPaymentMethodData,
    OutboundPaymentCreateParamsDestinationPaymentMethodDataBillingDetails as OutboundPaymentCreateParamsDestinationPaymentMethodDataBillingDetails,
    OutboundPaymentCreateParamsDestinationPaymentMethodDataBillingDetailsAddress as OutboundPaymentCreateParamsDestinationPaymentMethodDataBillingDetailsAddress,
    OutboundPaymentCreateParamsDestinationPaymentMethodDataUsBankAccount as OutboundPaymentCreateParamsDestinationPaymentMethodDataUsBankAccount,
    OutboundPaymentCreateParamsDestinationPaymentMethodOptions as OutboundPaymentCreateParamsDestinationPaymentMethodOptions,
    OutboundPaymentCreateParamsDestinationPaymentMethodOptionsUsBankAccount as OutboundPaymentCreateParamsDestinationPaymentMethodOptionsUsBankAccount,
    OutboundPaymentCreateParamsEndUserDetails as OutboundPaymentCreateParamsEndUserDetails,
)
from stripe.params.treasury._outbound_payment_fail_params import (
    OutboundPaymentFailParams as OutboundPaymentFailParams,
)
from stripe.params.treasury._outbound_payment_list_params import (
    OutboundPaymentListParams as OutboundPaymentListParams,
    OutboundPaymentListParamsCreated as OutboundPaymentListParamsCreated,
)
from stripe.params.treasury._outbound_payment_post_params import (
    OutboundPaymentPostParams as OutboundPaymentPostParams,
)
from stripe.params.treasury._outbound_payment_retrieve_params import (
    OutboundPaymentRetrieveParams as OutboundPaymentRetrieveParams,
)
from stripe.params.treasury._outbound_payment_return_outbound_payment_params import (
    OutboundPaymentReturnOutboundPaymentParams as OutboundPaymentReturnOutboundPaymentParams,
    OutboundPaymentReturnOutboundPaymentParamsReturnedDetails as OutboundPaymentReturnOutboundPaymentParamsReturnedDetails,
)
from stripe.params.treasury._outbound_payment_update_params import (
    OutboundPaymentUpdateParams as OutboundPaymentUpdateParams,
    OutboundPaymentUpdateParamsTrackingDetails as OutboundPaymentUpdateParamsTrackingDetails,
    OutboundPaymentUpdateParamsTrackingDetailsAch as OutboundPaymentUpdateParamsTrackingDetailsAch,
    OutboundPaymentUpdateParamsTrackingDetailsUsDomesticWire as OutboundPaymentUpdateParamsTrackingDetailsUsDomesticWire,
)
from stripe.params.treasury._outbound_transfer_cancel_params import (
    OutboundTransferCancelParams as OutboundTransferCancelParams,
)
from stripe.params.treasury._outbound_transfer_create_params import (
    OutboundTransferCreateParams as OutboundTransferCreateParams,
    OutboundTransferCreateParamsDestinationPaymentMethodData as OutboundTransferCreateParamsDestinationPaymentMethodData,
    OutboundTransferCreateParamsDestinationPaymentMethodOptions as OutboundTransferCreateParamsDestinationPaymentMethodOptions,
    OutboundTransferCreateParamsDestinationPaymentMethodOptionsUsBankAccount as OutboundTransferCreateParamsDestinationPaymentMethodOptionsUsBankAccount,
    OutboundTransferCreateParamsNetworkDetails as OutboundTransferCreateParamsNetworkDetails,
    OutboundTransferCreateParamsNetworkDetailsAch as OutboundTransferCreateParamsNetworkDetailsAch,
)
from stripe.params.treasury._outbound_transfer_fail_params import (
    OutboundTransferFailParams as OutboundTransferFailParams,
)
from stripe.params.treasury._outbound_transfer_list_params import (
    OutboundTransferListParams as OutboundTransferListParams,
)
from stripe.params.treasury._outbound_transfer_post_params import (
    OutboundTransferPostParams as OutboundTransferPostParams,
)
from stripe.params.treasury._outbound_transfer_retrieve_params import (
    OutboundTransferRetrieveParams as OutboundTransferRetrieveParams,
)
from stripe.params.treasury._outbound_transfer_return_outbound_transfer_params import (
    OutboundTransferReturnOutboundTransferParams as OutboundTransferReturnOutboundTransferParams,
    OutboundTransferReturnOutboundTransferParamsReturnedDetails as OutboundTransferReturnOutboundTransferParamsReturnedDetails,
)
from stripe.params.treasury._outbound_transfer_update_params import (
    OutboundTransferUpdateParams as OutboundTransferUpdateParams,
    OutboundTransferUpdateParamsTrackingDetails as OutboundTransferUpdateParamsTrackingDetails,
    OutboundTransferUpdateParamsTrackingDetailsAch as OutboundTransferUpdateParamsTrackingDetailsAch,
    OutboundTransferUpdateParamsTrackingDetailsUsDomesticWire as OutboundTransferUpdateParamsTrackingDetailsUsDomesticWire,
)
from stripe.params.treasury._received_credit_create_params import (
    ReceivedCreditCreateParams as ReceivedCreditCreateParams,
    ReceivedCreditCreateParamsInitiatingPaymentMethodDetails as ReceivedCreditCreateParamsInitiatingPaymentMethodDetails,
    ReceivedCreditCreateParamsInitiatingPaymentMethodDetailsUsBankAccount as ReceivedCreditCreateParamsInitiatingPaymentMethodDetailsUsBankAccount,
    ReceivedCreditCreateParamsNetworkDetails as ReceivedCreditCreateParamsNetworkDetails,
    ReceivedCreditCreateParamsNetworkDetailsAch as ReceivedCreditCreateParamsNetworkDetailsAch,
)
from stripe.params.treasury._received_credit_list_params import (
    ReceivedCreditListParams as ReceivedCreditListParams,
    ReceivedCreditListParamsLinkedFlows as ReceivedCreditListParamsLinkedFlows,
)
from stripe.params.treasury._received_credit_retrieve_params import (
    ReceivedCreditRetrieveParams as ReceivedCreditRetrieveParams,
)
from stripe.params.treasury._received_debit_create_params import (
    ReceivedDebitCreateParams as ReceivedDebitCreateParams,
    ReceivedDebitCreateParamsInitiatingPaymentMethodDetails as ReceivedDebitCreateParamsInitiatingPaymentMethodDetails,
    ReceivedDebitCreateParamsInitiatingPaymentMethodDetailsUsBankAccount as ReceivedDebitCreateParamsInitiatingPaymentMethodDetailsUsBankAccount,
    ReceivedDebitCreateParamsNetworkDetails as ReceivedDebitCreateParamsNetworkDetails,
    ReceivedDebitCreateParamsNetworkDetailsAch as ReceivedDebitCreateParamsNetworkDetailsAch,
)
from stripe.params.treasury._received_debit_list_params import (
    ReceivedDebitListParams as ReceivedDebitListParams,
)
from stripe.params.treasury._received_debit_retrieve_params import (
    ReceivedDebitRetrieveParams as ReceivedDebitRetrieveParams,
)
from stripe.params.treasury._transaction_entry_list_params import (
    TransactionEntryListParams as TransactionEntryListParams,
    TransactionEntryListParamsCreated as TransactionEntryListParamsCreated,
    TransactionEntryListParamsEffectiveAt as TransactionEntryListParamsEffectiveAt,
)
from stripe.params.treasury._transaction_entry_retrieve_params import (
    TransactionEntryRetrieveParams as TransactionEntryRetrieveParams,
)
from stripe.params.treasury._transaction_list_params import (
    TransactionListParams as TransactionListParams,
    TransactionListParamsCreated as TransactionListParamsCreated,
    TransactionListParamsStatusTransitions as TransactionListParamsStatusTransitions,
    TransactionListParamsStatusTransitionsPostedAt as TransactionListParamsStatusTransitionsPostedAt,
)
from stripe.params.treasury._transaction_retrieve_params import (
    TransactionRetrieveParams as TransactionRetrieveParams,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.params.v2.billing.intents._action_list_params import (
    ActionListParams as ActionListParams,
)
from stripe.params.v2.billing.intents._action_retrieve_params import (
    ActionRetrieveParams as ActionRetrieveParams,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2.billing.pricing_plans._component_service import (
    ComponentService as ComponentService,
)
from stripe.v2.billing.pricing_plans._version_service import (
    VersionService as VersionService,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._list_object import ListObject
from stripe.v2.billing._collection_setting import CollectionSetting
from stripe.v2.billing.collection_settings._version_service import (
    VersionService,
)
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.billing._collection_setting_create_params import (
        CollectionSettingCreateParams,
    )
    from stripe.params.v2.billing._collection_setting_list_params import (
        CollectionSettingListParams,
    )
    from stripe.params.v2.billing._collection_setting_retrieve_params import (
        CollectionSettingRetrieveParams,
    )
    from stripe.params.v2.billing._collection_setting_update_params import (
        CollectionSettingUpdateParams,
    )


class CollectionSettingService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.versions = VersionService(self._requestor)

    def list(
        self,
        params: Optional["CollectionSettingListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[CollectionSetting]:
        """
        List all CollectionSetting objects.
        """
        return cast(
            ListObject[CollectionSetting],
            self._request(
                "get",
                "/v2/billing/collection_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["CollectionSettingListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[CollectionSetting]:
        """
        List all CollectionSetting objects.
        """
        return cast(
            ListObject[CollectionSetting],
            await self._request_async(
                "get",
                "/v2/billing/collection_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["CollectionSettingCreateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> CollectionSetting:
        """
        Create a CollectionSetting object.
        """
        return cast(
            CollectionSetting,
            self._request(
                "post",
                "/v2/billing/collection_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["CollectionSettingCreateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> CollectionSetting:
        """
        Create a CollectionSetting object.
        """
        return cast(
            CollectionSetting,
            await self._request_async(
                "post",
                "/v2/billing/collection_settings",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["CollectionSettingRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> CollectionSetting:
        """
        Retrieve a CollectionSetting by ID.
        """
        return cast(
            CollectionSetting,
            self._request(
                "get",
                "/v2/billing/collection_settings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["CollectionSettingRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> CollectionSetting:
        """
        Retrieve a CollectionSetting by ID.
        """
        return cast(
            CollectionSetting,
            await self._request_async(
                "get",
                "/v2/billing/collection_settings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["CollectionSettingUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> CollectionSetting:
        """
        Update fields on an existing CollectionSetting.
        """
        return cast(
            CollectionSetting,
            self._request(
                "post",
                "/v2/billing/collection_settings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["CollectionSettingUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> CollectionSetting:
        """
        Update fields on an existing CollectionSetting.
        """
        return cast(
            CollectionSetting,
            await self._request_async(
                "post",
                "/v2/billing/collection_settings/{id}".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._list_object import ListObject
from stripe.v2.billing._intent import Intent
from stripe.v2.billing.intents._action_service import ActionService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.billing._intent_cancel_params import (
        IntentCancelParams,
    )
    from stripe.params.v2.billing._intent_commit_params import (
        IntentCommitParams,
    )
    from stripe.params.v2.billing._intent_create_params import (
        IntentCreateParams,
    )
    from stripe.params.v2.billing._intent_list_params import IntentListParams
    from stripe.params.v2.billing._intent_release_reservation_params import (
        IntentReleaseReservationParams,
    )
    from stripe.params.v2.billing._intent_reserve_params import (
        IntentReserveParams,
    )
    from stripe.params.v2.billing._intent_retrieve_params import (
        IntentRetrieveParams,
    )


class IntentService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.actions = ActionService(self._requestor)

    def list(
        self,
        params: Optional["IntentListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[Intent]:
        """
        List Billing Intents.
        """
        return cast(
            ListObject[Intent],
            self._request(
                "get",
                "/v2/billing/intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["IntentListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[Intent]:
        """
        List Billing Intents.
        """
        return cast(
            ListObject[Intent],
            await self._request_async(
                "get",
                "/v2/billing/intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "IntentCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Create a Billing Intent.
        """
        return cast(
            Intent,
            self._request(
                "post",
                "/v2/billing/intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "IntentCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Create a Billing Intent.
        """
        return cast(
            Intent,
            await self._request_async(
                "post",
                "/v2/billing/intents",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["IntentRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Retrieve a Billing Intent.
        """
        return cast(
            Intent,
            self._request(
                "get",
                "/v2/billing/intents/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["IntentRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Retrieve a Billing Intent.
        """
        return cast(
            Intent,
            await self._request_async(
                "get",
                "/v2/billing/intents/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        id: str,
        params: Optional["IntentCancelParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Cancel a Billing Intent.
        """
        return cast(
            Intent,
            self._request(
                "post",
                "/v2/billing/intents/{id}/cancel".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        id: str,
        params: Optional["IntentCancelParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Cancel a Billing Intent.
        """
        return cast(
            Intent,
            await self._request_async(
                "post",
                "/v2/billing/intents/{id}/cancel".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def commit(
        self,
        id: str,
        params: Optional["IntentCommitParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Commit a Billing Intent.
        """
        return cast(
            Intent,
            self._request(
                "post",
                "/v2/billing/intents/{id}/commit".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def commit_async(
        self,
        id: str,
        params: Optional["IntentCommitParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Commit a Billing Intent.
        """
        return cast(
            Intent,
            await self._request_async(
                "post",
                "/v2/billing/intents/{id}/commit".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def release_reservation(
        self,
        id: str,
        params: Optional["IntentReleaseReservationParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Release a Billing Intent.
        """
        return cast(
            Intent,
            self._request(
                "post",
                "/v2/billing/intents/{id}/release_reservation".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def release_reservation_async(
        self,
        id: str,
        params: Optional["IntentReleaseReservationParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Release a Billing Intent.
        """
        return cast(
            Intent,
            await self._request_async(
                "post",
                "/v2/billing/intents/{id}/release_reservation".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def reserve(
        self,
        id: str,
        params: Optional["IntentReserveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Reserve a Billing Intent.
        """
        return cast(
            Intent,
            self._request(
                "post",
                "/v2/billing/intents/{id}/reserve".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def reserve_async(
        self,
        id: str,
        params: Optional["IntentReserveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Intent:
        """
        Reserve a Billing Intent.
        """
        return cast(
            Intent,
            await self._request_async(
                "post",
                "/v2/billing/intents/{id}/reserve".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
from stripe.v2.core._event import (
    EventNotification as EventNotification,
    RelatedObject as RelatedObject,
    Reason as Reason,
    ReasonRequest as ReasonRequest,
)

# The beginning of the section generated from our OpenAPI spec
from stripe.v2.core import accounts as accounts, vault as vault
from stripe.v2.core._account import Account as Account
from stripe.v2.core._account_link import AccountLink as AccountLink
from stripe.v2.core._account_link_service import (
    AccountLinkService as AccountLinkService,
)
from stripe.v2.core._account_person import AccountPerson as AccountPerson
from stripe.v2.core._account_service import AccountService as AccountService
from stripe.v2.core._claimable_sandbox import (
    ClaimableSandbox as ClaimableSandbox,
)
from stripe.v2.core._claimable_sandbox_service import (
    ClaimableSandboxService as ClaimableSandboxService,
)
from stripe.v2.core._event import Event as Event
from stripe.v2.core._event_destination import (
    EventDestination as EventDestination,
)
from stripe.v2.core._event_destination_service import (
    EventDestinationService as EventDestinationService,
)
from stripe.v2.core._event_service import EventService as EventService
from stripe.v2.core._vault_service import VaultService as VaultService
# The end of the section generated from our OpenAPI spec



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2.tax._automatic_rule import AutomaticRule
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.tax._automatic_rule_create_params import (
        AutomaticRuleCreateParams,
    )
    from stripe.params.v2.tax._automatic_rule_deactivate_params import (
        AutomaticRuleDeactivateParams,
    )
    from stripe.params.v2.tax._automatic_rule_find_params import (
        AutomaticRuleFindParams,
    )
    from stripe.params.v2.tax._automatic_rule_retrieve_params import (
        AutomaticRuleRetrieveParams,
    )
    from stripe.params.v2.tax._automatic_rule_update_params import (
        AutomaticRuleUpdateParams,
    )


class AutomaticRuleService(StripeService):
    def create(
        self,
        params: "AutomaticRuleCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Creates an AutomaticRule object.
        """
        return cast(
            AutomaticRule,
            self._request(
                "post",
                "/v2/tax/automatic_rules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "AutomaticRuleCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Creates an AutomaticRule object.
        """
        return cast(
            AutomaticRule,
            await self._request_async(
                "post",
                "/v2/tax/automatic_rules",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def find(
        self,
        params: "AutomaticRuleFindParams",
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Finds an AutomaticRule object by BillableItem ID.
        """
        return cast(
            AutomaticRule,
            self._request(
                "get",
                "/v2/tax/automatic_rules/find",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def find_async(
        self,
        params: "AutomaticRuleFindParams",
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Finds an AutomaticRule object by BillableItem ID.
        """
        return cast(
            AutomaticRule,
            await self._request_async(
                "get",
                "/v2/tax/automatic_rules/find",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["AutomaticRuleRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Retrieves an AutomaticRule object by ID.
        """
        return cast(
            AutomaticRule,
            self._request(
                "get",
                "/v2/tax/automatic_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["AutomaticRuleRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Retrieves an AutomaticRule object by ID.
        """
        return cast(
            AutomaticRule,
            await self._request_async(
                "get",
                "/v2/tax/automatic_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: "AutomaticRuleUpdateParams",
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Updates the automatic Tax configuration for an AutomaticRule object.
        """
        return cast(
            AutomaticRule,
            self._request(
                "post",
                "/v2/tax/automatic_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: "AutomaticRuleUpdateParams",
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Updates the automatic Tax configuration for an AutomaticRule object.
        """
        return cast(
            AutomaticRule,
            await self._request_async(
                "post",
                "/v2/tax/automatic_rules/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def deactivate(
        self,
        id: str,
        params: Optional["AutomaticRuleDeactivateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Deactivates an AutomaticRule object. Deactivated AutomaticRule objects are ignored in future tax calculations.
        """
        return cast(
            AutomaticRule,
            self._request(
                "post",
                "/v2/tax/automatic_rules/{id}/deactivate".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def deactivate_async(
        self,
        id: str,
        params: Optional["AutomaticRuleDeactivateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> AutomaticRule:
        """
        Deactivates an AutomaticRule object. Deactivated AutomaticRule objects are ignored in future tax calculations.
        """
        return cast(
            AutomaticRule,
            await self._request_async(
                "post",
                "/v2/tax/automatic_rules/{id}/deactivate".format(
                    id=sanitize_id(id),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.params.test_helpers.treasury._inbound_transfer_fail_params import (
    InboundTransferFailParams as InboundTransferFailParams,
    InboundTransferFailParamsFailureDetails as InboundTransferFailParamsFailureDetails,
)
from stripe.params.test_helpers.treasury._inbound_transfer_return_inbound_transfer_params import (
    InboundTransferReturnInboundTransferParams as InboundTransferReturnInboundTransferParams,
)
from stripe.params.test_helpers.treasury._inbound_transfer_succeed_params import (
    InboundTransferSucceedParams as InboundTransferSucceedParams,
)
from stripe.params.test_helpers.treasury._outbound_payment_fail_params import (
    OutboundPaymentFailParams as OutboundPaymentFailParams,
)
from stripe.params.test_helpers.treasury._outbound_payment_post_params import (
    OutboundPaymentPostParams as OutboundPaymentPostParams,
)
from stripe.params.test_helpers.treasury._outbound_payment_return_outbound_payment_params import (
    OutboundPaymentReturnOutboundPaymentParams as OutboundPaymentReturnOutboundPaymentParams,
    OutboundPaymentReturnOutboundPaymentParamsReturnedDetails as OutboundPaymentReturnOutboundPaymentParamsReturnedDetails,
)
from stripe.params.test_helpers.treasury._outbound_payment_update_params import (
    OutboundPaymentUpdateParams as OutboundPaymentUpdateParams,
    OutboundPaymentUpdateParamsTrackingDetails as OutboundPaymentUpdateParamsTrackingDetails,
    OutboundPaymentUpdateParamsTrackingDetailsAch as OutboundPaymentUpdateParamsTrackingDetailsAch,
    OutboundPaymentUpdateParamsTrackingDetailsUsDomesticWire as OutboundPaymentUpdateParamsTrackingDetailsUsDomesticWire,
)
from stripe.params.test_helpers.treasury._outbound_transfer_fail_params import (
    OutboundTransferFailParams as OutboundTransferFailParams,
)
from stripe.params.test_helpers.treasury._outbound_transfer_post_params import (
    OutboundTransferPostParams as OutboundTransferPostParams,
)
from stripe.params.test_helpers.treasury._outbound_transfer_return_outbound_transfer_params import (
    OutboundTransferReturnOutboundTransferParams as OutboundTransferReturnOutboundTransferParams,
    OutboundTransferReturnOutboundTransferParamsReturnedDetails as OutboundTransferReturnOutboundTransferParamsReturnedDetails,
)
from stripe.params.test_helpers.treasury._outbound_transfer_update_params import (
    OutboundTransferUpdateParams as OutboundTransferUpdateParams,
    OutboundTransferUpdateParamsTrackingDetails as OutboundTransferUpdateParamsTrackingDetails,
    OutboundTransferUpdateParamsTrackingDetailsAch as OutboundTransferUpdateParamsTrackingDetailsAch,
    OutboundTransferUpdateParamsTrackingDetailsUsDomesticWire as OutboundTransferUpdateParamsTrackingDetailsUsDomesticWire,
)
from stripe.params.test_helpers.treasury._received_credit_create_params import (
    ReceivedCreditCreateParams as ReceivedCreditCreateParams,
    ReceivedCreditCreateParamsInitiatingPaymentMethodDetails as ReceivedCreditCreateParamsInitiatingPaymentMethodDetails,
    ReceivedCreditCreateParamsInitiatingPaymentMethodDetailsUsBankAccount as ReceivedCreditCreateParamsInitiatingPaymentMethodDetailsUsBankAccount,
    ReceivedCreditCreateParamsNetworkDetails as ReceivedCreditCreateParamsNetworkDetails,
    ReceivedCreditCreateParamsNetworkDetailsAch as ReceivedCreditCreateParamsNetworkDetailsAch,
)
from stripe.params.test_helpers.treasury._received_debit_create_params import (
    ReceivedDebitCreateParams as ReceivedDebitCreateParams,
    ReceivedDebitCreateParamsInitiatingPaymentMethodDetails as ReceivedDebitCreateParamsInitiatingPaymentMethodDetails,
    ReceivedDebitCreateParamsInitiatingPaymentMethodDetailsUsBankAccount as ReceivedDebitCreateParamsInitiatingPaymentMethodDetailsUsBankAccount,
    ReceivedDebitCreateParamsNetworkDetails as ReceivedDebitCreateParamsNetworkDetails,
    ReceivedDebitCreateParamsNetworkDetailsAch as ReceivedDebitCreateParamsNetworkDetailsAch,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._account_link import AccountLink
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._account_link_create_params import (
        AccountLinkCreateParams,
    )


class AccountLinkService(StripeService):
    def create(
        self,
        params: "AccountLinkCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> AccountLink:
        """
        Creates an AccountLink object that includes a single-use Stripe URL that the platform can redirect their user to in order to take them through the Connect Onboarding flow.
        """
        return cast(
            AccountLink,
            self._request(
                "post",
                "/v1/account_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "AccountLinkCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> AccountLink:
        """
        Creates an AccountLink object that includes a single-use Stripe URL that the platform can redirect their user to in order to take them through the Connect Onboarding flow.
        """
        return cast(
            AccountLink,
            await self._request_async(
                "post",
                "/v1/account_links",
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.v2.test_helpers._financial_address_service import (
    FinancialAddressService as FinancialAddressService,
)
from stripe.v2.test_helpers._money_management_service import (
    MoneyManagementService as MoneyManagementService,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from stripe.v2._list_object import ListObject
from stripe.v2.billing._rate_card import RateCard
from stripe.v2.billing.rate_cards._rate_service import RateService
from stripe.v2.billing.rate_cards._version_service import VersionService
from typing import Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params.v2.billing._rate_card_create_params import (
        RateCardCreateParams,
    )
    from stripe.params.v2.billing._rate_card_list_params import (
        RateCardListParams,
    )
    from stripe.params.v2.billing._rate_card_retrieve_params import (
        RateCardRetrieveParams,
    )
    from stripe.params.v2.billing._rate_card_update_params import (
        RateCardUpdateParams,
    )


class RateCardService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.rates = RateService(self._requestor)
        self.versions = VersionService(self._requestor)

    def list(
        self,
        params: Optional["RateCardListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[RateCard]:
        """
        List all Rate Card objects.
        """
        return cast(
            ListObject[RateCard],
            self._request(
                "get",
                "/v2/billing/rate_cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["RateCardListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[RateCard]:
        """
        List all Rate Card objects.
        """
        return cast(
            ListObject[RateCard],
            await self._request_async(
                "get",
                "/v2/billing/rate_cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: "RateCardCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> RateCard:
        """
        Create a Rate Card object.
        """
        return cast(
            RateCard,
            self._request(
                "post",
                "/v2/billing/rate_cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: "RateCardCreateParams",
        options: Optional[RequestOptions] = None,
    ) -> RateCard:
        """
        Create a Rate Card object.
        """
        return cast(
            RateCard,
            await self._request_async(
                "post",
                "/v2/billing/rate_cards",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        id: str,
        params: Optional["RateCardRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> RateCard:
        """
        Retrieve the latest version of a Rate Card object.
        """
        return cast(
            RateCard,
            self._request(
                "get",
                "/v2/billing/rate_cards/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        id: str,
        params: Optional["RateCardRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> RateCard:
        """
        Retrieve the latest version of a Rate Card object.
        """
        return cast(
            RateCard,
            await self._request_async(
                "get",
                "/v2/billing/rate_cards/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        id: str,
        params: Optional["RateCardUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> RateCard:
        """
        Update a Rate Card object.
        """
        return cast(
            RateCard,
            self._request(
                "post",
                "/v2/billing/rate_cards/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        id: str,
        params: Optional["RateCardUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> RateCard:
        """
        Update a Rate Card object.
        """
        return cast(
            RateCard,
            await self._request_async(
                "post",
                "/v2/billing/rate_cards/{id}".format(id=sanitize_id(id)),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe.params.v2.money_management._adjustment_list_params import (
    AdjustmentListParams as AdjustmentListParams,
)
from stripe.params.v2.money_management._adjustment_retrieve_params import (
    AdjustmentRetrieveParams as AdjustmentRetrieveParams,
)
from stripe.params.v2.money_management._financial_account_close_params import (
    FinancialAccountCloseParams as FinancialAccountCloseParams,
    FinancialAccountCloseParamsForwardingSettings as FinancialAccountCloseParamsForwardingSettings,
)
from stripe.params.v2.money_management._financial_account_create_params import (
    FinancialAccountCreateParams as FinancialAccountCreateParams,
    FinancialAccountCreateParamsStorage as FinancialAccountCreateParamsStorage,
)
from stripe.params.v2.money_management._financial_account_list_params import (
    FinancialAccountListParams as FinancialAccountListParams,
)
from stripe.params.v2.money_management._financial_account_retrieve_params import (
    FinancialAccountRetrieveParams as FinancialAccountRetrieveParams,
)
from stripe.params.v2.money_management._financial_address_create_params import (
    FinancialAddressCreateParams as FinancialAddressCreateParams,
    FinancialAddressCreateParamsSepaBankAccount as FinancialAddressCreateParamsSepaBankAccount,
)
from stripe.params.v2.money_management._financial_address_list_params import (
    FinancialAddressListParams as FinancialAddressListParams,
)
from stripe.params.v2.money_management._financial_address_retrieve_params import (
    FinancialAddressRetrieveParams as FinancialAddressRetrieveParams,
)
from stripe.params.v2.money_management._inbound_transfer_create_params import (
    InboundTransferCreateParams as InboundTransferCreateParams,
    InboundTransferCreateParamsFrom as InboundTransferCreateParamsFrom,
    InboundTransferCreateParamsTo as InboundTransferCreateParamsTo,
)
from stripe.params.v2.money_management._inbound_transfer_list_params import (
    InboundTransferListParams as InboundTransferListParams,
)
from stripe.params.v2.money_management._inbound_transfer_retrieve_params import (
    InboundTransferRetrieveParams as InboundTransferRetrieveParams,
)
from stripe.params.v2.money_management._outbound_payment_cancel_params import (
    OutboundPaymentCancelParams as OutboundPaymentCancelParams,
)
from stripe.params.v2.money_management._outbound_payment_create_params import (
    OutboundPaymentCreateParams as OutboundPaymentCreateParams,
    OutboundPaymentCreateParamsDeliveryOptions as OutboundPaymentCreateParamsDeliveryOptions,
    OutboundPaymentCreateParamsFrom as OutboundPaymentCreateParamsFrom,
    OutboundPaymentCreateParamsRecipientNotification as OutboundPaymentCreateParamsRecipientNotification,
    OutboundPaymentCreateParamsTo as OutboundPaymentCreateParamsTo,
)
from stripe.params.v2.money_management._outbound_payment_list_params import (
    OutboundPaymentListParams as OutboundPaymentListParams,
)
from stripe.params.v2.money_management._outbound_payment_quote_create_params import (
    OutboundPaymentQuoteCreateParams as OutboundPaymentQuoteCreateParams,
    OutboundPaymentQuoteCreateParamsDeliveryOptions as OutboundPaymentQuoteCreateParamsDeliveryOptions,
    OutboundPaymentQuoteCreateParamsFrom as OutboundPaymentQuoteCreateParamsFrom,
    OutboundPaymentQuoteCreateParamsTo as OutboundPaymentQuoteCreateParamsTo,
)
from stripe.params.v2.money_management._outbound_payment_quote_retrieve_params import (
    OutboundPaymentQuoteRetrieveParams as OutboundPaymentQuoteRetrieveParams,
)
from stripe.params.v2.money_management._outbound_payment_retrieve_params import (
    OutboundPaymentRetrieveParams as OutboundPaymentRetrieveParams,
)
from stripe.params.v2.money_management._outbound_setup_intent_cancel_params import (
    OutboundSetupIntentCancelParams as OutboundSetupIntentCancelParams,
)
from stripe.params.v2.money_management._outbound_setup_intent_create_params import (
    OutboundSetupIntentCreateParams as OutboundSetupIntentCreateParams,
    OutboundSetupIntentCreateParamsPayoutMethodData as OutboundSetupIntentCreateParamsPayoutMethodData,
    OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount as OutboundSetupIntentCreateParamsPayoutMethodDataBankAccount,
    OutboundSetupIntentCreateParamsPayoutMethodDataCard as OutboundSetupIntentCreateParamsPayoutMethodDataCard,
    OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet as OutboundSetupIntentCreateParamsPayoutMethodDataCryptoWallet,
)
from stripe.params.v2.money_management._outbound_setup_intent_list_params import (
    OutboundSetupIntentListParams as OutboundSetupIntentListParams,
)
from stripe.params.v2.money_management._outbound_setup_intent_retrieve_params import (
    OutboundSetupIntentRetrieveParams as OutboundSetupIntentRetrieveParams,
)
from stripe.params.v2.money_management._outbound_setup_intent_update_params import (
    OutboundSetupIntentUpdateParams as OutboundSetupIntentUpdateParams,
    OutboundSetupIntentUpdateParamsPayoutMethodData as OutboundSetupIntentUpdateParamsPayoutMethodData,
    OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount as OutboundSetupIntentUpdateParamsPayoutMethodDataBankAccount,
    OutboundSetupIntentUpdateParamsPayoutMethodDataCard as OutboundSetupIntentUpdateParamsPayoutMethodDataCard,
)
from stripe.params.v2.money_management._outbound_transfer_cancel_params import (
    OutboundTransferCancelParams as OutboundTransferCancelParams,
)
from stripe.params.v2.money_management._outbound_transfer_create_params import (
    OutboundTransferCreateParams as OutboundTransferCreateParams,
    OutboundTransferCreateParamsDeliveryOptions as OutboundTransferCreateParamsDeliveryOptions,
    OutboundTransferCreateParamsFrom as OutboundTransferCreateParamsFrom,
    OutboundTransferCreateParamsTo as OutboundTransferCreateParamsTo,
)
from stripe.params.v2.money_management._outbound_transfer_list_params import (
    OutboundTransferListParams as OutboundTransferListParams,
)
from stripe.params.v2.money_management._outbound_transfer_retrieve_params import (
    OutboundTransferRetrieveParams as OutboundTransferRetrieveParams,
)
from stripe.params.v2.money_management._payout_method_archive_params import (
    PayoutMethodArchiveParams as PayoutMethodArchiveParams,
)
from stripe.params.v2.money_management._payout_method_list_params import (
    PayoutMethodListParams as PayoutMethodListParams,
    PayoutMethodListParamsUsageStatus as PayoutMethodListParamsUsageStatus,
)
from stripe.params.v2.money_management._payout_method_retrieve_params import (
    PayoutMethodRetrieveParams as PayoutMethodRetrieveParams,
)
from stripe.params.v2.money_management._payout_method_unarchive_params import (
    PayoutMethodUnarchiveParams as PayoutMethodUnarchiveParams,
)
from stripe.params.v2.money_management._payout_methods_bank_account_spec_retrieve_params import (
    PayoutMethodsBankAccountSpecRetrieveParams as PayoutMethodsBankAccountSpecRetrieveParams,
)
from stripe.params.v2.money_management._received_credit_list_params import (
    ReceivedCreditListParams as ReceivedCreditListParams,
)
from stripe.params.v2.money_management._received_credit_retrieve_params import (
    ReceivedCreditRetrieveParams as ReceivedCreditRetrieveParams,
)
from stripe.params.v2.money_management._received_debit_list_params import (
    ReceivedDebitListParams as ReceivedDebitListParams,
)
from stripe.params.v2.money_management._received_debit_retrieve_params import (
    ReceivedDebitRetrieveParams as ReceivedDebitRetrieveParams,
)
from stripe.params.v2.money_management._recipient_verification_acknowledge_params import (
    RecipientVerificationAcknowledgeParams as RecipientVerificationAcknowledgeParams,
)
from stripe.params.v2.money_management._recipient_verification_create_params import (
    RecipientVerificationCreateParams as RecipientVerificationCreateParams,
)
from stripe.params.v2.money_management._recipient_verification_retrieve_params import (
    RecipientVerificationRetrieveParams as RecipientVerificationRetrieveParams,
)
from stripe.params.v2.money_management._transaction_entry_list_params import (
    TransactionEntryListParams as TransactionEntryListParams,
)
from stripe.params.v2.money_management._transaction_entry_retrieve_params import (
    TransactionEntryRetrieveParams as TransactionEntryRetrieveParams,
)
from stripe.params.v2.money_management._transaction_list_params import (
    TransactionListParams as TransactionListParams,
)
from stripe.params.v2.money_management._transaction_retrieve_params import (
    TransactionRetrieveParams as TransactionRetrieveParams,
)



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from stripe._invoice_line_item import InvoiceLineItem
from stripe._list_object import ListObject
from stripe._quote import Quote
from stripe._quote_computed_upfront_line_items_service import (
    QuoteComputedUpfrontLineItemsService,
)
from stripe._quote_line_item_service import QuoteLineItemService
from stripe._quote_line_service import QuoteLineService
from stripe._quote_preview_invoice_service import QuotePreviewInvoiceService
from stripe._quote_preview_subscription_schedule_service import (
    QuotePreviewSubscriptionScheduleService,
)
from stripe._request_options import RequestOptions
from stripe._stripe_service import StripeService
from stripe._util import sanitize_id
from typing import Any, Optional, cast
from typing_extensions import TYPE_CHECKING

if TYPE_CHECKING:
    from stripe.params._quote_accept_params import QuoteAcceptParams
    from stripe.params._quote_cancel_params import QuoteCancelParams
    from stripe.params._quote_create_params import QuoteCreateParams
    from stripe.params._quote_finalize_quote_params import (
        QuoteFinalizeQuoteParams,
    )
    from stripe.params._quote_list_params import QuoteListParams
    from stripe.params._quote_list_preview_invoice_lines_params import (
        QuoteListPreviewInvoiceLinesParams,
    )
    from stripe.params._quote_mark_draft_params import QuoteMarkDraftParams
    from stripe.params._quote_mark_stale_params import QuoteMarkStaleParams
    from stripe.params._quote_pdf_params import QuotePdfParams
    from stripe.params._quote_reestimate_params import QuoteReestimateParams
    from stripe.params._quote_retrieve_params import QuoteRetrieveParams
    from stripe.params._quote_update_params import QuoteUpdateParams


class QuoteService(StripeService):
    def __init__(self, requestor):
        super().__init__(requestor)
        self.computed_upfront_line_items = (
            QuoteComputedUpfrontLineItemsService(
                self._requestor,
            )
        )
        self.lines = QuoteLineService(self._requestor)
        self.line_items = QuoteLineItemService(self._requestor)
        self.preview_invoices = QuotePreviewInvoiceService(self._requestor)
        self.preview_subscription_schedules = (
            QuotePreviewSubscriptionScheduleService(
                self._requestor,
            )
        )

    def list(
        self,
        params: Optional["QuoteListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[Quote]:
        """
        Returns a list of your quotes.
        """
        return cast(
            ListObject[Quote],
            self._request(
                "get",
                "/v1/quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_async(
        self,
        params: Optional["QuoteListParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[Quote]:
        """
        Returns a list of your quotes.
        """
        return cast(
            ListObject[Quote],
            await self._request_async(
                "get",
                "/v1/quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def create(
        self,
        params: Optional["QuoteCreateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        A quote models prices and services for a customer. Default options for header, description, footer, and expires_at can be set in the dashboard via the [quote template](https://dashboard.stripe.com/settings/billing/quote).
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def create_async(
        self,
        params: Optional["QuoteCreateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        A quote models prices and services for a customer. Default options for header, description, footer, and expires_at can be set in the dashboard via the [quote template](https://dashboard.stripe.com/settings/billing/quote).
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes",
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def retrieve(
        self,
        quote: str,
        params: Optional["QuoteRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Retrieves the quote with the given ID.
        """
        return cast(
            Quote,
            self._request(
                "get",
                "/v1/quotes/{quote}".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def retrieve_async(
        self,
        quote: str,
        params: Optional["QuoteRetrieveParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Retrieves the quote with the given ID.
        """
        return cast(
            Quote,
            await self._request_async(
                "get",
                "/v1/quotes/{quote}".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def update(
        self,
        quote: str,
        params: Optional["QuoteUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        A quote models prices and services for a customer.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def update_async(
        self,
        quote: str,
        params: Optional["QuoteUpdateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        A quote models prices and services for a customer.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def accept(
        self,
        quote: str,
        params: Optional["QuoteAcceptParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Accepts the specified quote.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}/accept".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def accept_async(
        self,
        quote: str,
        params: Optional["QuoteAcceptParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Accepts the specified quote.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}/accept".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def cancel(
        self,
        quote: str,
        params: Optional["QuoteCancelParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Cancels the quote.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}/cancel".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def cancel_async(
        self,
        quote: str,
        params: Optional["QuoteCancelParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Cancels the quote.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}/cancel".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def finalize_quote(
        self,
        quote: str,
        params: Optional["QuoteFinalizeQuoteParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Finalizes the quote.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}/finalize".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def finalize_quote_async(
        self,
        quote: str,
        params: Optional["QuoteFinalizeQuoteParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Finalizes the quote.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}/finalize".format(quote=sanitize_id(quote)),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def mark_draft(
        self,
        quote: str,
        params: Optional["QuoteMarkDraftParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Converts a stale quote to draft.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}/mark_draft".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def mark_draft_async(
        self,
        quote: str,
        params: Optional["QuoteMarkDraftParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Converts a stale quote to draft.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}/mark_draft".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def mark_stale(
        self,
        quote: str,
        params: Optional["QuoteMarkStaleParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Converts a draft or open quote to stale.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}/mark_stale".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def mark_stale_async(
        self,
        quote: str,
        params: Optional["QuoteMarkStaleParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Converts a draft or open quote to stale.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}/mark_stale".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def reestimate(
        self,
        quote: str,
        params: Optional["QuoteReestimateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Recompute the upcoming invoice estimate for the quote.
        """
        return cast(
            Quote,
            self._request(
                "post",
                "/v1/quotes/{quote}/reestimate".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def reestimate_async(
        self,
        quote: str,
        params: Optional["QuoteReestimateParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Quote:
        """
        Recompute the upcoming invoice estimate for the quote.
        """
        return cast(
            Quote,
            await self._request_async(
                "post",
                "/v1/quotes/{quote}/reestimate".format(
                    quote=sanitize_id(quote),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    def pdf(
        self,
        quote: str,
        params: Optional["QuotePdfParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Any:
        """
        Download the PDF for a finalized quote. Explanation for special handling can be found [here](https://docs.stripe.com/quotes/overview#quote_pdf)
        """
        return cast(
            Any,
            self._request_stream(
                "get",
                "/v1/quotes/{quote}/pdf".format(quote=sanitize_id(quote)),
                base_address="files",
                params=params,
                options=options,
            ),
        )

    async def pdf_async(
        self,
        quote: str,
        params: Optional["QuotePdfParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> Any:
        """
        Download the PDF for a finalized quote. Explanation for special handling can be found [here](https://docs.stripe.com/quotes/overview#quote_pdf)
        """
        return cast(
            Any,
            await self._request_stream_async(
                "get",
                "/v1/quotes/{quote}/pdf".format(quote=sanitize_id(quote)),
                base_address="files",
                params=params,
                options=options,
            ),
        )

    def list_preview_invoice_lines(
        self,
        quote: str,
        preview_invoice: str,
        params: Optional["QuoteListPreviewInvoiceLinesParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[InvoiceLineItem]:
        """
        Preview the invoice line items that would be generated by accepting the quote.
        """
        return cast(
            ListObject[InvoiceLineItem],
            self._request(
                "get",
                "/v1/quotes/{quote}/preview_invoices/{preview_invoice}/lines".format(
                    quote=sanitize_id(quote),
                    preview_invoice=sanitize_id(preview_invoice),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )

    async def list_preview_invoice_lines_async(
        self,
        quote: str,
        preview_invoice: str,
        params: Optional["QuoteListPreviewInvoiceLinesParams"] = None,
        options: Optional[RequestOptions] = None,
    ) -> ListObject[InvoiceLineItem]:
        """
        Preview the invoice line items that would be generated by accepting the quote.
        """
        return cast(
            ListObject[InvoiceLineItem],
            await self._request_async(
                "get",
                "/v1/quotes/{quote}/preview_invoices/{preview_invoice}/lines".format(
                    quote=sanitize_id(quote),
                    preview_invoice=sanitize_id(preview_invoice),
                ),
                base_address="api",
                params=params,
                options=options,
            ),
        )



# -*- coding: utf-8 -*-
# File generated from our OpenAPI spec
from typing_extensions import TypedDict


class AutomaticRuleCreateParams(TypedDict):
    billable_item: str
    """
    The BillableItem ID to set automatic Tax configuration for.
    """
    tax_code: str
    """
    The TaxCode object to be used for automatic tax calculations.
    """
